#include <stdlib.h>
#include <malloc.h>
#include "mpi.h"


/*	Bestimmt Min und Max (incl. Indizes) des Feldes x	*/

void MinMaxD(n,x,min,kmin,max,kmax)

int n,*kmin,*kmax;
double x[], *min, *max;
{
 int i;
 
 *max = *min = x[0];
 *kmax = *kmin = 0;
 for (i=1; i<n; i++ )
   {
     if (x[i]>*max) { *max = x[i]; *kmax = i;}
       else  if (x[i]<*min) { *min = x[i]; *kmin = i;}
   }
}


/*				Exchange glob. Min. and Max. of a vector */

void PivotD(n,x)

int    n;
double x[];
{
/*			mm[0] - loc. minimum ,  mm[1] - loc. maximum	*/
/*			km : local indices				*/
/*			gm[0] - glob. minimum , gm[1] - glob. maximum	*/
/*			gmid : process-id with min/max			*/

 int    km[2], gmid[2], root=0, numprocs, myid, i ;
 double mm[2], gm[2], *mg, tt;
 MPI_Status stat;
 
 MPI_Comm_rank(MPI_COMM_WORLD,&myid);

 MinMaxD(n,x,&mm[0],&km[0],&mm[1],&km[1]);	/* local min/max 	*/
 /*	printf("\n %g  %d   %g  %d \n",mm[0],km[0],mm[1],km[1]);	*/

 if (myid==root)  				/* require aux. array	*/
   { 
     MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
     mg = (double*)malloc(2*numprocs*sizeof(double));
   }

 MPI_Gather(mm,2,MPI_DOUBLE,mg,2,MPI_DOUBLE,root,MPI_COMM_WORLD);
 
 /*	DebugD(2*numprocs,mg);	*/

 if (myid==root) 			/* root determines glob. Min/Max */
  { 				
   gm[0] = mg[0]; gmid[0] = 0;			/*  global min 	*/
   for (i=1; i<numprocs; i++) 
     if ( mg[2*i] < gm[0] ) { gm[0]=mg[2*i]; gmid[0]=i; }
 				
   gm[1] = mg[1]; gmid[1] = 0;			/*  global max 	*/
   for (i=1; i<numprocs; i++) 
     if ( mg[2*i+1] > gm[1] ) { gm[1]=mg[2*i+1]; gmid[1]=i; }
     
      /* printf("\nroot :  %g  %d   %g  %d \n",gm[0],gmid[0],gm[1],gmid[1]); */ 
  }
  
/*			Broadcast the processes with Min or Max	*/

 MPI_Bcast(gmid,2,MPI_INT,root,MPI_COMM_WORLD);
 
 
/*		gmid[0] and gmid[1]  exchange the Min or Max, resp.	*/

 if (gmid[0]==gmid[1] && myid==gmid[0]) 	/* just local exchange */
  { tt = x[km[0]];  x[km[0]] = x[km[1]];  x[km[1]] = tt; return; }

 if (myid == gmid[0]) 
  {
   MPI_Send(&x[km[0]],1,MPI_DOUBLE,gmid[1],0,MPI_COMM_WORLD);
   MPI_Recv(&x[km[0]],1,MPI_DOUBLE,gmid[1],1,MPI_COMM_WORLD,&stat);
  }
  
 if (myid == gmid[1])
  {
   tt = x[km[1]];
   MPI_Recv(&x[km[1]],1,MPI_DOUBLE,gmid[0],0,MPI_COMM_WORLD,&stat);
   MPI_Send(&tt,1,MPI_DOUBLE,gmid[0],1,MPI_COMM_WORLD);
  } 

 if (myid==root)  				/* require aux. array	*/
   { 
     free(mg);
   }
 
}
